<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output method="html" indent="yes"/>
	<xsl:template match="/">
		<html lang="en-us">
			<head>
				<title>Assertion Coverage Report</title>
				<link rel="stylesheet" type="text/css"
					href="http://www.w3.org/StyleSheets/TR/base.css"/>
				<meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>
				<style type="text/css"> .border th, .border td { border: 1px solid black; padding:
					0.5ex } .border th { background: #ccc } .good { background: green } .bad {
					background: red } .class, .count { text-align: center } .bar { font-weight:
					bold; } .synopsis { font-size: small } 
					.warning {background-color:yellow}</style>
			</head>
			<body>
				<h1>Assertion Coverage Report</h1>
				<table>
					<tr>
						<td><b><xsl:value-of select="count(assertions/assertion)"/> assertions</b></td>
						<td>
							<xsl:call-template name="assertion-summary"/>
						</td>
					</tr>
					<tr>
						<td>Part 1 (<xsl:value-of select="assertions/@part1"/> assertions)</td>
						<td>
							<xsl:call-template name="assertion-summary-by-part">
								<xsl:with-param name="part" select="1"/>
								<xsl:with-param name="width" select="assertions/@part1 div count(assertions/assertion)"/>
							</xsl:call-template>
						</td>
					</tr>
					<tr>
						<td>Part 2 (<xsl:value-of select="assertions/@part2"/> assertions)</td>
						<td><xsl:call-template name="assertion-summary-by-part">
							<xsl:with-param name="part" select="2"/>
							<xsl:with-param name="width" select="assertions/@part2 div count(assertions/assertion)"/>
						</xsl:call-template></td>
					</tr>
				</table>
				<p>Assertion breakdown: <xsl:value-of select="assertions/@component"/> component, 
					<xsl:value-of select="assertions/@document"/> document, 
					<xsl:value-of select="assertions/@exchange"	/> exchange, 
					<xsl:value-of select="assertions/@message"/> message. 
				</p>
				<p>Testcase complexity: <xsl:value-of select="assertions/@one-violation"/>
					testcases violate 1 assertion,
					<xsl:value-of select="assertions/@two-violations"/>
					testcases violate 2 assertions, and
					<xsl:value-of select="assertions/@more-violations"/>
					testcases violate 3 or more assertions.<br/> 
				</p>
				<xsl:if test="assertions/unknown-assertion-ids/assertion">
					<p class="warning">
						<xsl:text>Warning! Unknown assertion numbers: </xsl:text> 
						<xsl:for-each select="assertions/unknown-assertion-ids/assertion">
							<a href="{@origin}"><xsl:value-of select="@id"/></a>
							<xsl:text>, </xsl:text>		
						</xsl:for-each>
						<xsl:text>defined in testcase metadata.</xsl:text>
					</p>
				</xsl:if>
				<table class="border">
					<thead>
						<tr>
							<th>Status</th>
							<th>Count</th>
							<th>Assertion</th>
							<th>Synopsis</th>
							<th>Class</th>
							<th>Tests</th>
						</tr>
					</thead>
					<tbody>
						<xsl:for-each select="assertions/assertion">
							<xsl:sort select="@name"/>
							<tr>
								<td>
									<xsl:attribute name="class">
										<xsl:choose>
											<xsl:when test="@count = 0">bad</xsl:when>
											<xsl:otherwise>good</xsl:otherwise>
										</xsl:choose>
									</xsl:attribute>
									<xsl:text>&#160;</xsl:text>
								</td>
								<td class="count">
									<xsl:value-of select="@count"/>
								</td>
								<td>
									<a href="{@href}">
										<xsl:value-of select="@name"/>
									</a>
								</td>
								<td class="synopsis">
									<xsl:value-of select="synopsis"/>
								</td>
								<td class="class">
									<xsl:value-of select="@class"/>
								</td>
								<td>
									<xsl:for-each select="test">
										<xsl:text>[</xsl:text>
										<a href="{@href}">
											<xsl:value-of select="@id"/>
										</a>
										<xsl:text>] </xsl:text>
									</xsl:for-each>
								</td>
							</tr>
						</xsl:for-each>
					</tbody>
				</table>
			</body>
		</html>
	</xsl:template>
	<xsl:template name="assertion-summary">
		<div style="border:1px solid black; padding:0px; width:202px; height:18px; font-size:x-small; font-weight:bold">
			<xsl:variable name="total" select="count(assertions/assertion)"/>
			<xsl:variable name="none" select="count(assertions/assertion[@count = 0])"/>
			<xsl:variable name="some" select="count(assertions/assertion[@count &gt; 0])"/>
			<xsl:variable name="percentage-none" select="$none div $total"/>
			<xsl:variable name="percentage-some" select="$some div $total"/>
			<xsl:if test="$percentage-some > .01">
				<span style="position:absolute">
					<xsl:value-of select="$some"/>
				</span>
				<img src="green.gif" height="16" width="{round(200*$percentage-some)}" alt="{round($percentage-some * 100)}% have coverage"/>
			</xsl:if>
			<xsl:if test="$percentage-none > .01">
				<span style="position:absolute">
					<xsl:value-of select="$none"/>
				</span>
				<img src="red.gif" height="16" width="{round(200*$percentage-none)}" alt="{round($percentage-none* 100)}% have no coverage"/>
			</xsl:if>
		</div>
	</xsl:template>
	<xsl:template name="assertion-summary-by-part">
		<xsl:param name="part"/>
		<xsl:param name="width" select="1.0"/>
		<div style="border:1px solid black; padding:0px; width:{round($width*200)+3}px; height:18px; font-size:x-small; font-weight:bold">
			<xsl:variable name="total" select="count(assertions/assertion[@part=$part])"/>
			<xsl:variable name="none" select="count(assertions/assertion[@part=$part][@count = 0])"/>
			<xsl:variable name="some" select="count(assertions/assertion[@part=$part][@count &gt; 0])"/>
			<xsl:variable name="percentage-none" select="$none div $total"/>
			<xsl:variable name="percentage-some" select="$some div $total"/>
			<xsl:if test="$percentage-some > .01">
				<span style="position:absolute">
					<xsl:value-of select="$some"/>
				</span>
				<img src="green.gif" height="16" width="{round($width*200*$percentage-some)}" alt="{round($percentage-some * 100)}% have coverage"/>
			</xsl:if>
			<xsl:if test="$percentage-none > .01">
				<span style="position:absolute">
					<xsl:value-of select="$none"/>
				</span>
				<img src="red.gif" height="16" width="{round($width*200*$percentage-none)}" alt="{round($percentage-none* 100)}% have no coverage"/>
			</xsl:if>
		</div>
	</xsl:template>
</xsl:stylesheet>
