<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

	<xsl:output method="xml" indent="yes" />

	<xsl:template match="/">
		<xsl:processing-instruction name="xml-stylesheet">type="text/xsl" href="assertion-coverage.xsl"</xsl:processing-instruction>
		<xsl:variable name="ViolatesAssertions" select="ViolatesAssertions" />
		<xsl:variable name="assertions"
					select="document('assertions.xml')/assertions/assertion"/>
        <xsl:variable name="count"
			            select="count($assertions)"/>
        <xsl:variable name="count_component"
			            select="count($assertions[@class='component'])"/>
        <xsl:variable name="count_document"
			            select="count($assertions[@class='document'])"/>
        <xsl:variable name="count_exchange"
			            select="count($assertions[@class='exchange'])"/>
        <xsl:variable name="count_message"
			            select="count($assertions[@class='message'])"/>

        <xsl:variable name="unique_violations" select="$ViolatesAssertions//ViolateAssertion[not(@assertion=following::ViolateAssertion/@assertion)]" />   
    
		<assertions component="{$count_component}" document="{$count_document}"
					exchange="{$count_exchange}" message="{$count_message}" 
					part1="{count($assertions[@part=1])}" part2="{count($assertions[@part=2])}"
					one-violation="{count($ViolatesAssertions/assertion-count[@number=1])}"
					two-violations="{count($ViolatesAssertions/assertion-count[@number=2])}"
					more-violations="{count($ViolatesAssertions/assertion-count[@number>2])}">
			<xsl:for-each select="$assertions">
				<xsl:sort select="@id"/>
				<xsl:variable name="id" select="@id"/>
				<xsl:variable name="violations" select="$ViolatesAssertions/ViolateAssertion[@assertion=$id]"/>
				<xsl:variable name="count_violations_2" select="count($violations)"/>
			
				<assertion name="{$id}" class="{@class}" href="{concat(@origin, '/#', @id)}" count="{$count_violations_2}" part="{@part}">
					<synopsis><xsl:copy-of select="node()"/></synopsis>
					<xsl:for-each select="$violations">
						<test href="{@origin}" id="{@test-id}"/>
					</xsl:for-each>
				</assertion>
				
			</xsl:for-each>

			<unknown-assertion-ids>
				<xsl:for-each select="$ViolatesAssertions/ViolateAssertion">
					<xsl:if test="not(@assertion = $assertions/@id)">
						<assertion id="{@assertion}" origin="{@origin}"/>
					</xsl:if>
				</xsl:for-each>
			</unknown-assertion-ids>
		</assertions>
	</xsl:template>
</xsl:stylesheet>
