<?xml version="1.0" encoding="UTF-8"?>
<!--
	Formats coverage report.
	
	ChangeLog:
	
	2006-07-29 Arthur Ryman <ryman@ca.ibm.com>
	- fixed heading
	
	2006-02-19 Arthur Ryman <ryman@ca.ibm.com>
	- added W3C css
	
	2005-11-11 Arthur Ryman <ryman@ca.ibm.com>
	- created
-->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	version="1.0">

	<xsl:output method="html" encoding="UTF-8" indent="yes" />

	<xsl:template match="/coverage">
		<html>
			<head>
				<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
				<title>
					WSDL 2.0 Document Test Case Coverage Report
				</title>
				<link rel="stylesheet" type="text/css"
					href="http://www.w3.org/StyleSheets/TR/base.css" />
			</head>
			<body>
				<h1>
					WSDL 2.0 Document Test Case Coverage Report
				</h1>
				<xsl:if test="TODAY">
					<address>
						<xsl:text>Generated </xsl:text>
						<xsl:value-of select="TODAY" />,
						<xsl:value-of select="TSTAMP" />
					</address>
				</xsl:if>
				<p>
					This reports measures the coverage of WSDL 2.0
					elements and attributes by the good and bad document
					test cases. Each measured item is specified by an
					XPath expression.
				</p>
				<table>
					<tr>
						<th align="right">Good Documents:</th>
						<td align="left">
							<xsl:value-of select="good" />
						</td>
					</tr>
					<tr>
						<th align="right">Bad Documents:</th>
						<td align="left">
							<xsl:value-of select="bad" />
						</td>
					</tr>
					<tr>
						<th align="right">Total Documents:</th>
						<td align="left">
							<xsl:value-of select="total" />
						</td>
					</tr>
					<tr>
						<th align="right">Coverage Summary:</th>
						<td>
							<xsl:call-template name="test-coverage-summary"/>
						</td>
					</tr>
				</table>
				<br />
				<table border="1" cellspacing="4" width="100%">
					<col width="50" />
					<col width="50" />
					<col width="*" />
					<thead>
						<tr>
							<th>Status</th>
							<th>Count</th>
							<th>XPath</th>
						</tr>
					</thead>
					<tbody>
						<xsl:apply-templates select="item" />
					</tbody>
				</table>
			</body>
		</html>
	</xsl:template>

	<xsl:template match="item">
		<tr>
			<td count="{@count}">
				<xsl:attribute name="bgcolor">
					<xsl:choose>
						<xsl:when test="@count = 0">red</xsl:when>
						<xsl:when test="@count &lt; 4">yellow</xsl:when>
						<xsl:otherwise>green</xsl:otherwise>
					</xsl:choose>
				</xsl:attribute>
				<xsl:text>&#160;</xsl:text>
			</td>
			<td align="right">
				<xsl:value-of select="@count" />
			</td>
			<td>
				<xsl:value-of select="@xpath" />
			</td>
		</tr>
	</xsl:template>
	<xsl:template name="test-coverage-summary">
		<div style="border:1px solid black; padding:0px; width:202px; height:18px; font-size:x-small; font-weight:bold; margin:0px">
			<xsl:variable name="total" select="count(item)"/>
			<xsl:variable name="none" select="count(item[@count = 0])"/>
			<xsl:variable name="some" select="count(item[@count &gt; 0 and @count &lt; 4])"/>
			<xsl:variable name="many" select="count(item[@count &gt; 3])"/>
			<xsl:variable name="percentage-none" select="$none div $total"/>
			<xsl:variable name="percentage-some" select="$some div $total"/>
			<xsl:variable name="percentage-many" select="$many div $total"/>
			<xsl:if test="$percentage-many > .01"><span style="position:absolute"><xsl:value-of select="round($percentage-many*100)"/>%</span><img src="green.gif" height="16" width="{round(200*$percentage-many)}" alt="{$many} have good coverage"/></xsl:if>
			<xsl:if test="$percentage-some > .01"><span style="position:absolute"><xsl:value-of select="round($percentage-some*100)"/>%</span><img src="yellow.gif" height="16" width="{round(200*$percentage-some)}" alt="{$some} have minimal coverage"/></xsl:if>
			<xsl:if test="$percentage-none > 0"><span style="position:absolute"><xsl:value-of select="round((1 - $percentage-many - $percentage-some)*100)"/>%</span><img src="red.gif" height="16" width="{200 - round(200*$percentage-many) - round(200*$percentage-some)}" alt="{$none} have no coverage"/></xsl:if>
		</div>
	</xsl:template>
</xsl:stylesheet>