<?xml version="1.0" encoding="UTF-8"?>
<?xml-stylesheet type="text/xsl" href="dashboard.xsl"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
	xmlns:report="http://www.w3.org/2006/06/wsdl/ValidationReport" >
	<xsl:import href="results/interchange.xsl"/>
	<xsl:import href="results/validation.xsl"/>
	<xsl:import href="assertion-coverage.xsl"/>
	<xsl:import href="coverage.xsl"/>
	<xsl:import href="results-messages/message-tests-results.xsl"/>
	
	<xsl:param name="DSTAMP" />
	<xsl:param name="TSTAMP" />
	<xsl:param name="TODAY" />
	
	<xsl:output method="html" encoding="UTF-8" indent="no"/>
	<xsl:template match="/">
		<html>
			<head>
				<title>WSDL 2.0 Interop Dashboard</title>
				<style type="text/css">
					body {font-family:sans-serif}
					td {vertical-align:top}
					.passed   {background-color:green}
					.failed   {background-color:orange}
					.disaster {background-color:red}
					
					.border th, .border td { border: 1px solid black; padding: 0.5ex }
					.border th { background: #ccc }
					.good { background: green }
					.bad { background: red }
					
					.group {background-color:#DDD}
					.heading {font-weight:bold; white-space: nowrap}
					.explanatory {font-size:smaller}
				</style>
			</head>
			<body>
				<h2>WSDL 2.0 Interop Dashboard</h2>
		
				<xsl:if test="$TODAY">
					<address>
						<xsl:text>Generated </xsl:text>
						<xsl:value-of select="$TODAY" />,
						<xsl:value-of select="$TSTAMP" />
					</address>
				</xsl:if>

				<p class="explanatory">This page summarizes the results of interop testing of the 
					<a href="http://www.w3.org/TR/wsdl20">WSDL 2.0 Core</a> and 
					<a href="http://www.w3.org/TR/wsdl20-adjuncts">WSDL 2.0 Adjuncts</a> specifications,
					in fulfillment of the <a href="http://lists.w3.org/Archives/Public/www-ws-desc/2005Nov/0036.html">CR exit criteria</a>.  
					The completeness of the
					<a href="http://dev.w3.org/cvsweb/~checkout~/2002/ws/desc/test-suite/index.html">Test Suite</a>
					is also measured.</p>
				<p class="explanatory">The results of interop testing prior to the namespace change can be found <a href="http://dev.w3.org/cvsweb/~checkout~/2002/ws/desc/test-suite/Dashboard.html?rev=1.75">here</a>.</p>
				<p class="explanatory">The interop testing is based on implementations from <a href="http://incubator.apache.org/woden/">Apache Woden</a> (a WSDL 2.0 parser), 
					<a href="http://www.wso2.com/products/wsas/">WSO2 WSAS</a> (a web services engine that uses Woden and is built on <a href="http://ws.apache.org/axis2/">Apache Axis2</a>) and 
					a WSDL 2.0 parser and web services engine from Canon."</p>
				<table cellspacing="3">
					<tr>
						<td colspan="4" class="group">
							<h3>Interop results</h3>
						</td>
					</tr>
					<tr>
						<td>
							<div class="heading">Component Model Test Results</div>
						</td>
						<td>
							<p class="explanatory">(<a href="results/Interchange.html">details</a>)</p>
						</td>
						<td colspan="2">
							<p class="explanatory">These results test the results of parsing a WSDL 2.0 document, by generating an 
								interchange format representing the component model in XML and comparing it against a baseline description
								of the component model.</p>
						</td>
					</tr>
					<tr>
						<td/>
						<td colspan="3">
							<xsl:for-each select="document('results/interchange.xml')">
								<xsl:call-template name="interchange-results-summary"/>
							</xsl:for-each>
						</td>
					</tr>
					<tr>
						<td colspan="4"><hr/></td>
					</tr>
					<tr>
						<td>
							<div class="heading">Message Exchange Test Results</div>
						</td>
						<td>
							<p class="explanatory">(<a href="results-messages/Message-tests-results.html">details</a>)</p>
						</td>
						<td colspan="2">
							<p class="explanatory">These results represent the analysis of log files of message exchanges based
							on complex WSDL 2.0 documents, with each implementation run pair-wise against the others.</p>
						</td>
					</tr>
					<tr>
						<td colspan="4">
							<xsl:for-each select="document('results-messages/message-tests-results.xml')">
								<xsl:call-template name="message-exchange-results-summary">
									<xsl:with-param name="detail-path" select="'results-messages/Message-tests-results.html'"/>
								</xsl:call-template>
							</xsl:for-each>
						</td>
					</tr>
					<tr>
						<td colspan="4"><hr/></td>
					</tr>
					<tr>
						<td>
							<div class="heading">Validation Test Results</div>
						</td>
						<td>
							<p class="explanatory">(<a href="results/Validation.html">details</a>)</p>
						</td>
						<td colspan="2">
							<p class="explanatory">These results represent the analysis of validation log files generated by validating implementations consuming both good and bad documents in the test suite.  Green and yellow both mean that the testcase was correctly validated, the difference being that a yellow indicates that the implementation did not correctly identify the specific assertion a "bad" testcase was designed fail on.</p>
						</td>
					</tr>
					<tr>
						<td/>
						<td colspan="3">
							<table>
								<xsl:for-each select="document('results/validation.xml')">
									<thead>
										<tr>
											<xsl:for-each select="summary/vendors/vendor">
												<td><strong><xsl:value-of select="@name"/></strong></td>
											</xsl:for-each>
										</tr>
									</thead>
									<tbody>
										<tr>
											<xsl:call-template name="validation-summaries"/>
										</tr>
									</tbody>
								</xsl:for-each>
							</table>
						</td>
					</tr>
					<tr>
						<td colspan="4"><hr/></td>
					</tr>
						
					<tr>
						<td colspan="4" class="group">
							<h3>Test Suite Completeness</h3>
						</td>
					</tr>
					<tr>
						<td>
							<div class="heading">Test Case Coverage</div>
						</td>
						<td>
							<p class="explanatory">(<a href="Test-suite-coverage-summary.html">details</a>)</p>
						</td>
						<td colspan="2">
							<p class="explanatory">An analysis of the use of various parts of the WSDL syntax (elements, attributes, uris) in the test collection.  Green indicates 4 or more occurances of the construct, yellow indicates less than 4 occurances, and red indicates zero occurances.</p>
						</td>
					</tr>
					<tr>
						<td/>
						<td colspan="3">
							<xsl:for-each select="document('test-suite-coverage-summary.xml')/coverage">
								<xsl:call-template name="test-coverage-summary"/>
							</xsl:for-each>		
						</td>
					</tr>
					<tr>
						<td colspan="4"><hr/></td>
					</tr>
					<tr>
						<td>
							<div class="heading">Assertion Coverage</div>
						</td>
						<td>
							<p class="explanatory">(<a href="Assertion-coverage.html">details</a>)</p>
						</td>
						<td colspan="2">
							<p class="explanatory">Count of assertions violated by "bad" testcases, compared with the total number of assertions
							identified in the specs.</p>
						</td>
					</tr>
					<tr>
						<td/>
						<td colspan="3">
							<xsl:for-each select="document('assertion-coverage.xml')">
								<xsl:call-template name="assertion-summary"/>
							</xsl:for-each>
						</td>
						
					</tr>
				</table>
				
			</body>
		</html>
	</xsl:template>

</xsl:stylesheet>