<?xml version="1.0" encoding="utf-8"?>
<!--

    $Header: /sources/public/2002/ws/desc/test-suite/results-messages/generate-message-properties.xslt,v 1.14 2007/03/23 05:50:53 jonathanMarsh Exp $

    Convert WSDL Component model into message properties

-->
<xsl:stylesheet version="1.0"
                xmlns="http://www.w3.org/2006/11/desc/messages"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:cm='http://www.w3.org/2002/ws/desc/wsdl/component'
                xmlns:cmextensions='http://www.w3.org/2002/ws/desc/wsdl/component-extensions'
                xmlns:cmbase='http://www.w3.org/2002/ws/desc/wsdl/component-base'
                xmlns:cmhttp='http://www.w3.org/2002/ws/desc/wsdl/component-http'
                xmlns:cmrpc='http://www.w3.org/2002/ws/desc/wsdl/component-rpc'
                xmlns:cmsoap='http://www.w3.org/2002/ws/desc/wsdl/component-soap'
                exclude-result-prefixes='cmsoap cmrpc cmhttp cmbase cmextensions cm'>

  <xsl:output method="xml" indent="yes" />

  <xsl:template match='/'>
    <messages>
      <xsl:call-template name='process-endpointComponents'>
        <xsl:with-param name='endpointComponents' select='/cm:descriptionComponent/cm:services/cm:serviceComponent/cm:endpoints/cm:endpointComponent'/>
        <xsl:with-param name='start'>0</xsl:with-param>
      </xsl:call-template>
    </messages>
  </xsl:template>

  <!-- Process a list of endpoint components -->
  <xsl:template name='process-endpointComponents'>
    <xsl:param name='endpointComponents'/>
    <!-- how many operations processed before this endpoint component list? -->
    <xsl:param name='start'/>


    <xsl:if test='count($endpointComponents) != 0'>

      <!-- process the first element of the list -->
      <xsl:call-template name='process-endpointComponent'>
        <xsl:with-param name='endpointComponent' select='$endpointComponents[position()=1]'/>
        <xsl:with-param name='start' select='$start'/>
      </xsl:call-template>

      <!-- count how many operations are attached to the endpoint -->
      <xsl:variable name='current'>
        <xsl:call-template name='count-messages-in-interfaceReferences'>
          <xsl:with-param name='interfaceReferences'
                          select='$endpointComponents[position()=1]/ancestor::cm:serviceComponent/cm:interface'/>
        </xsl:call-template>
      </xsl:variable>

      <!-- process the remainder of the list -->
      <xsl:call-template name='process-endpointComponents'>
        <xsl:with-param name='endpointComponents' select='$endpointComponents[position()>1]'/>
        <xsl:with-param name='start' select='$start + $current'/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <xsl:template name='process-endpointComponent'>
    <xsl:param name='endpointComponent' />
    <xsl:param name='start' />

    <xsl:call-template name='process-interfaceReferences'>
      <xsl:with-param name='interfaceReferences'
                      select='$endpointComponent/ancestor::cm:serviceComponent/cm:interface'/>
      <xsl:with-param name='endpointComponent'
                      select='$endpointComponent'/>
      <xsl:with-param name='start' select='$start'/>
    </xsl:call-template>
    
  </xsl:template>

  <!-- Process each interface operation component for a given list of interface[@ref] -->
  <xsl:template name='process-interfaceReferences'>
    <!-- list of interface[@ref] -->
    <xsl:param name='interfaceReferences' />
    <xsl:param name='endpointComponent' />
    <xsl:param name='start' />
    
    <xsl:choose>
      <xsl:when test='count($interfaceReferences) = 0'>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name='interfaceComponent'
                      select='//cm:interfaceComponent[@xml:id = $interfaceReferences[position()=1]/@ref ]' />

        <xsl:variable name='extended'>
          <xsl:call-template name='count-messages-in-interfaceReferences'>
            <xsl:with-param name='interfaceReferences' select='$interfaceComponent/cm:extendedInterfaces/cm:interface'/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:variable name='current'>
          <xsl:call-template name='count-messages-in-interfaceOperationComponents'>
            <xsl:with-param name='interfaceOperationComponents'
                            select='$interfaceComponent/cm:interfaceOperations/cm:interfaceOperationComponent'/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:call-template name='process-interfaceReferences'>
          <xsl:with-param name='interfaceReferences' select='$interfaceComponent/cm:extendedInterfaces/cm:interface'/>
          <xsl:with-param name='endpointComponent' select='$endpointComponent' />
          <xsl:with-param name='start' select='$start' />
        </xsl:call-template>
        
        <xsl:call-template name='process-interfaceOperationComponents'>
          <xsl:with-param name='interfaceOperationComponents'
                          select='$interfaceComponent/cm:interfaceOperations/cm:interfaceOperationComponent'/>
          <xsl:with-param name='endpointComponent' select='$endpointComponent' />
          <xsl:with-param name='start' select='$start + $extended' />
        </xsl:call-template>

        <xsl:call-template name='process-interfaceReferences'>
          <xsl:with-param name='interfaceReferences' select='$interfaceReferences[position()>1]'/>
          <xsl:with-param name='endpointComponent' select='$endpointComponent' />
          <xsl:with-param name='start' select='$start + $extended + $current' />
        </xsl:call-template>

      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name='process-interfaceOperationComponents'>
    <xsl:param name='interfaceOperationComponents' />
    <xsl:param name='endpointComponent' />
    <xsl:param name='start' />

    <xsl:variable name='interfaceOperationComponent'
                  select='$interfaceOperationComponents[position()=1]'/>
    <xsl:variable name='mep' select='$interfaceOperationComponent/cm:messageExchangePattern'/>

    <!-- Process the input message -->
    <message>
      <xsl:call-template name='process-interfaceMessageReferenceComponent'>
        <xsl:with-param name='interfaceMessageReferenceComponent'
                        select='$interfaceOperationComponent/cm:interfaceMessageReferences/cm:interfaceMessageReferenceComponent[cm:direction="in"]' />
        <xsl:with-param name='endpointComponent' select='$endpointComponent' />
        <xsl:with-param name='position' select='$start + 1' />
      </xsl:call-template>
    </message>

    <!-- Process the output or fault message -->    
    <message>
      <xsl:choose>
        <xsl:when test='contains($interfaceOperationComponent/cm:name/cmbase:localName, "Fault")'>
          <!-- We have a fault message to process. @@Could have more than one.. -->
          <xsl:call-template name='process-interfaceFaultReferenceComponent'>
            <xsl:with-param name='interfaceFaultReferenceComponent'
			    select='$interfaceOperationComponent/cm:interfaceFaultReferences/cm:interfaceFaultReferenceComponent[cm:direction="out"][1]' />
            <xsl:with-param name='endpointComponent' select='$endpointComponent' />
            <xsl:with-param name='position' select='$start + 2' />
          </xsl:call-template>
        </xsl:when>
        <xsl:when test='$mep = "http://www.w3.org/ns/wsdl/in-out"'>
          <!-- We have a normal output message to process -->
          <xsl:call-template name='process-interfaceMessageReferenceComponent'>
            <xsl:with-param name='interfaceMessageReferenceComponent'
                            select='$interfaceOperationComponent/cm:interfaceMessageReferences/cm:interfaceMessageReferenceComponent[cm:direction="out"]' />
            <xsl:with-param name='endpointComponent' select='$endpointComponent' />
            <xsl:with-param name='position' select='$start + 2' />
          </xsl:call-template>
        </xsl:when>
        <xsl:when test='$mep = "http://www.w3.org/ns/wsdl/in-only"
                        or $mep = "http://www.w3.org/ns/wsdl/robust-in-only"'>
          <!-- We have an empty output message to process -->
          <xsl:call-template name='process-interfaceOperationComponent-EmptyOutput'>
            <xsl:with-param name='interfaceOperationComponent'
                            select='$interfaceOperationComponent' />
            <xsl:with-param name='endpointComponent' select='$endpointComponent' />
            <xsl:with-param name='position' select='$start + 2' />
          </xsl:call-template>
        </xsl:when>
      </xsl:choose>
    </message>

    <!-- Process the rest of the operations -->

    <xsl:if test='count($interfaceOperationComponents) > 1'>
      <xsl:variable name='current'>
        <xsl:call-template name='count-messages-in-interfaceOperationComponents'>
          <xsl:with-param name='interfaceOperationComponents'
                          select='$interfaceOperationComponent'/>
        </xsl:call-template>
      </xsl:variable>

      <xsl:call-template name='process-interfaceOperationComponents'>
        <xsl:with-param name='interfaceOperationComponents'
                        select='$interfaceOperationComponents[position()>1]'/>
        <xsl:with-param name='endpointComponent' select='$endpointComponent' />
        <xsl:with-param name='start' select='$start + $current' />
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <xsl:template name='process-interfaceMessageReferenceComponent'>
    <xsl:param name='interfaceMessageReferenceComponent' />
    <xsl:param name='endpointComponent'/>
    <xsl:param name='position' />

    <xsl:call-template name='compute-message-properties'>
      <xsl:with-param name='interfaceOperationComponent' select='$interfaceMessageReferenceComponent/../..'/>
      <xsl:with-param name='interfaceMessageReferenceComponent' select='$interfaceMessageReferenceComponent'/>
      <xsl:with-param name='endpointComponent'  select='$endpointComponent'/>
      <xsl:with-param name='position' select='$position' />
    </xsl:call-template>
  </xsl:template>

  <xsl:template name='process-interfaceFaultReferenceComponent'>
    <xsl:param name='interfaceFaultReferenceComponent' />
    <xsl:param name='endpointComponent'/>
    <xsl:param name='position' />

    <xsl:call-template name='compute-message-properties'>
      <xsl:with-param name='interfaceOperationComponent' select='$interfaceFaultReferenceComponent/../..'/>
      <xsl:with-param name='interfaceMessageReferenceComponent' select='$interfaceFaultReferenceComponent'/>
      <xsl:with-param name='endpointComponent'  select='$endpointComponent'/>
      <xsl:with-param name='position' select='$position' />
    </xsl:call-template>
  </xsl:template>

  <xsl:template name='process-interfaceOperationComponent-EmptyOutput'>
    <xsl:param name='interfaceOperationComponent' />
    <xsl:param name='endpointComponent'/>
    <xsl:param name='position' />

    <xsl:call-template name='compute-message-properties'>
      <xsl:with-param name='interfaceOperationComponent' select='$interfaceOperationComponent'/>
      <xsl:with-param name='interfaceMessageReferenceComponent' select='foo' />
      <xsl:with-param name='endpointComponent'  select='$endpointComponent'/>
      <xsl:with-param name='position' select='$position' />
    </xsl:call-template>
  </xsl:template>

  <xsl:template name='compute-message-properties'>
    <xsl:param name='interfaceOperationComponent' />
    <!-- this one could be empty (if in-only or robust-in-only without a fault) -->
    <xsl:param name='interfaceMessageReferenceComponent' />
    <xsl:param name='endpointComponent' />
    <xsl:param name='position' />

    <xsl:attribute name='messagePosition'>
      <xsl:value-of select='$position' />
    </xsl:attribute>
    <xsl:attribute name='type'>
      <xsl:choose>
        <xsl:when test='$interfaceMessageReferenceComponent/cm:direction="in"'>request</xsl:when>
        <xsl:when test='$interfaceMessageReferenceComponent/cm:direction="out"'>response</xsl:when>
        <xsl:otherwise>response</xsl:otherwise>
      </xsl:choose>
    </xsl:attribute>

    <xsl:variable name='bindingComponent'
                  select='//cm:bindingComponent[@xml:id = $endpointComponent/cm:binding/@ref]' />
    <xsl:variable name='bindingOperationComponent'
                  select='$bindingComponent/cm:bindingOperations/cm:bindingOperationComponent[cm:interfaceOperation[@ref=$interfaceOperationComponent/@xml:id]]' />
    <xsl:variable name='bindingMessageReferenceComponent'
                  select='$bindingOperationComponent/cm:bindingMessageReferences/cm:bindingMessageReferenceComponent[cm:interfaceMessageReference[@ref=$interfaceMessageReferenceComponent/@xml:id]]'/>
    <xsl:variable name='interfaceFaultComponent' 
                  select='//cm:interfaceFaultComponent[@xml:id = $interfaceMessageReferenceComponent/cm:interfaceFault/@ref]'/>
                  
    <xsl:variable name='bindingFaultComponent'
                  select='$bindingComponent/cm:bindingFaults/cm:bindingFaultComponent[cm:interfaceFault[@ref=$interfaceFaultComponent/@xml:id]]' />
    <xsl:variable name='bindingFaultReferenceComponent'
                  select='$bindingOperationComponent/cm:bindingFaultReferences/cm:bindingFaultReferenceComponent[cm:interfaceFaultReference[@ref=$interfaceFaultComponent/@xml:id]]' />

  	<endpoint name='{$endpointComponent/cm:name}' />
    <binding localName='{$bindingComponent/cm:name/cmbase:localName}'
             namespace='{$bindingComponent/cm:name/cmbase:namespaceName}'
             type='{$bindingComponent/cm:type}' />
    <interface localName='{$interfaceOperationComponent/../../cm:name/cmbase:localName}'
               namespace='{$interfaceOperationComponent/../../cm:name/cmbase:namespaceName}'/>
    <operation localName='{$interfaceOperationComponent/cm:name/cmbase:localName}'
               namespace='{$interfaceOperationComponent/cm:name/cmbase:namespaceName}'
               mep='{$interfaceOperationComponent/cm:messageExchangePattern}' />

    <xsl:if test='$endpointComponent/cm:address'>
      <address><xsl:value-of select='$endpointComponent/cm:address' /></address>
    </xsl:if>
    <xsl:for-each select='$interfaceOperationComponent/cm:style/cmbase:uri'>
      <style>
	<xsl:value-of select='.'/>
      </style>
    </xsl:for-each>
    <xsl:if test='$interfaceOperationComponent/cmextensions:wsdlInterfaceOperationExtension/cmextensions:safety'>
      <safety>
	<xsl:value-of select='$interfaceOperationComponent/cmextensions:wsdlInterfaceOperationExtension/cmextensions:safety'/>
      </safety>
    </xsl:if>
    <xsl:choose>
    <xsl:when test='$bindingComponent/cm:type="http://www.w3.org/ns/wsdl/soap"'>
      <xsl:if test='$bindingComponent/cmsoap:soapBindingExtension/cmhttp:httpCookies'>
        <httpCookies>
          <xsl:value-of select='$bindingComponent/cmsoap:soapBindingExtension/cmhttp:httpCookies'/>
        </httpCookies>
      </xsl:if>
      <xsl:if test='$bindingOperationComponent/cmsoap:soapBindingOperationExtension/cmhttp:httpLocation'>
        <httpLocation>
          <xsl:call-template name="replaceHttpLocation">
            <xsl:with-param name="location" select='$bindingOperationComponent/cmsoap:soapBindingOperationExtension/cmhttp:httpLocation' />
          </xsl:call-template>
        </httpLocation>
      </xsl:if>
      <xsl:choose>
        <xsl:when test='$bindingMessageReferenceComponent/cmsoap:soapBindingMessageReferenceExtension/cmhttp:httpContentEncoding'>
          <httpContentEncoding>
            <xsl:value-of select='$bindingMessageReferenceComponent/cmsoap:soapBindingMessageReferenceExtension/cmhttp:httpContentEncoding'/>
          </httpContentEncoding>
        </xsl:when>
        <xsl:when test='$bindingOperationComponent/cmsoap:soapBindingOperationExtension/cmhttp:httpContentEncodingDefault'>
          <httpContentEncoding>
            <xsl:value-of select='$bindingOperationComponent/cmsoap:soapBindingOperationExtension/cmhttp:httpContentEncodingDefault'/>
          </httpContentEncoding>
        </xsl:when>
        <xsl:when test='$bindingComponent/cmsoap:soapBindingExtension/cmhttp:httpContentEncodingDefault'>
          <httpContentEncoding>
            <xsl:value-of select='$bindingComponent/cmsoap:soapBindingExtension/cmhttp:httpContentEncodingDefault'/>
          </httpContentEncoding>
        </xsl:when>
      </xsl:choose>
      <xsl:if test='$endpointComponent/cmsoap:soapEndpointExtension/cmhttp:httpAuthenticationRealm'>
        <httpAuthenticationRealm><xsl:value-of select='$endpointComponent/cmsoap:soapEndpointExtension/cmhttp:httpAuthenticationRealm' /></httpAuthenticationRealm>
      </xsl:if>
      <xsl:if test='$endpointComponent/cmsoap:soapEndpointExtension/cmhttp:httpAuthenticationScheme'>
        <httpAuthenticationScheme><xsl:value-of select='$endpointComponent/cmsoap:soapEndpointExtension/cmhttp:httpAuthenticationScheme' /></httpAuthenticationScheme>
      </xsl:if>
      <soapUnderlyingProtocol>
	<xsl:value-of select='$bindingComponent/cmsoap:soapBindingExtension/cmsoap:soapUnderlyingProtocol' />
      </soapUnderlyingProtocol>
      <soapVersion>
        <xsl:value-of select='$bindingComponent/cmsoap:soapBindingExtension/cmsoap:soapVersion'/>
      </soapVersion>
      <soapMep>
        <xsl:if test='$interfaceOperationComponent/cm:messageExchangePattern="http://www.w3.org/ns/wsdl/in-out"
                      or $interfaceOperationComponent/cm:messageExchangePattern="http://www.w3.org/ns/wsdl/in-only"
                      or $interfaceOperationComponent/cm:messageExchangePattern="http://www.w3.org/ns/wsdl/robust-in-only"'>
          
          <xsl:choose>
            <xsl:when test='$bindingOperationComponent/cmsoap:soapBindingOperationExtension/cmsoap:soapMep'>
              <xsl:value-of select='$bindingOperationComponent/cmsoap:soapBindingOperationExtension/cmsoap:soapMep'/>
            </xsl:when>
            <xsl:when test='$bindingComponent/cmsoap:soapBindingExtension/cmsoap:soapMepDefault'>
              <xsl:value-of select='$bindingComponent/cmsoap:soapBindingExtension/cmsoap:soapMepDefault'/>
            </xsl:when>
            <xsl:when test='$bindingComponent/cmsoap:soapBindingExtension/cmsoap:soapVersion="1.2"'>
              <xsl:text>http://www.w3.org/2003/05/soap/mep/request-response/</xsl:text>            
            </xsl:when>
          </xsl:choose>
        </xsl:if>
      </soapMep>
      <xsl:if test='$interfaceMessageReferenceComponent'>
        <xsl:if test='$interfaceMessageReferenceComponent/cm:direction="in"
                      and $bindingOperationComponent/cmsoap:soapBindingOperationExtension/cmsoap:soapAction'>
          <soapAction>
            <xsl:value-of select='$bindingOperationComponent/cmsoap:soapBindingOperationExtension/cmsoap:soapAction'/>
          </soapAction>
        </xsl:if>
        <xsl:for-each select='$bindingMessageReferenceComponent/cmsoap:soapBindingMessageReferenceExtension/cmsoap:soapHeaders/cmsoap:soapHeaderBlockComponent'>
          <xsl:call-template name='generate-soap-header'>
            <xsl:with-param name='soapHeader' select='.' />
            <xsl:with-param name='elementDeclarationComponent' select='//cm:elementDeclarationComponent[@xml:id = current()/cmsoap:elementDeclaration/@ref]'/>
          </xsl:call-template>
        </xsl:for-each>
        <xsl:for-each select='$bindingMessageReferenceComponent/cmsoap:soapBindingMessageReferenceExtension/cmhttp:httpHeaders/cmhttp:httpHeaderComponent'>
          <xsl:call-template name='generate-http-header'>
            <xsl:with-param name='httpHeader' select='.' />
          </xsl:call-template>
        </xsl:for-each>
        <xsl:for-each select='$bindingMessageReferenceComponent/cmsoap:soapBindingFaultReferenceExtension/cmsoap:soapHeaders/cmsoap:soapHeaderBlockComponent'>
          <xsl:call-template name='generate-soap-header'>
            <xsl:with-param name='soapHeader' select='.' />
            <xsl:with-param name='elementDeclarationComponent' select='/cm:descriptionComponent//cm:elementDeclarationComponent[@xml:id = current()/cmsoap:elementDeclaration/@ref]'/>
          </xsl:call-template>
        </xsl:for-each>
        <xsl:for-each select='$bindingMessageReferenceComponent/cmsoap:soapBindingMessageReferenceExtension/cmsoap:soapModules/cmsoap:soapModuleComponent'>
          <soapModule required='{./cmsoap:required}'
                      ref ='{./cmsoap:ref}' />
        </xsl:for-each>
        <xsl:for-each select='$bindingOperationComponent/cmsoap:soapBindingOperationExtension/cmsoap:soapModules/cmsoap:soapModuleComponent'>
          <soapModule required='{./cmsoap:required}'
                      ref='{./cmsoap:ref}' />
        </xsl:for-each>
        <xsl:for-each select='$bindingComponent/cmsoap:soapBindingOperationExtension/cmsoap:soapModules/cmsoap:soapModuleComponent'>
          <soapModule required='{./cmsoap:required}'
                      ref='{./cmsoap:ref}' />
        </xsl:for-each>
        <xsl:choose>
          <xsl:when test='local-name($interfaceMessageReferenceComponent)="interfaceFaultReferenceComponent"'>
            <!-- @@position()=1 isn't correct -->
            <xsl:for-each select='$bindingFaultReferenceComponent[position()=1]/cmsoap:soapBindingFaultReferenceExtension/cmsoap:soapModules/cmsoap:soapModuleComponent'>
              <soapModule required='{./cmsoap:required}' ref ='{./cmsoap:ref}' />
            </xsl:for-each>
            <xsl:for-each select='$bindingMessageReferenceComponent/cmsoap:soapBindingFaultReferenceExtension/cmsoap:soapModules/cmsoap:soapModuleComponent'>
              <soapModule required='{./cmsoap:required}' ref ='{./cmsoap:ref}' />
            </xsl:for-each>
            <xsl:for-each select='$bindingFaultComponent/cmsoap:soapBindingFaultExtension/cmsoap:soapModules/cmsoap:soapModuleComponent'>
              <soapModule required='{./cmsoap:required}' ref ='{./cmsoap:ref}' />
            </xsl:for-each>
            <xsl:for-each select='$bindingFaultComponent/cmsoap:soapBindingFaultExtension/cmhttp:httpHeaders/cmhttp:httpHeaderComponent'>
              <xsl:call-template name='generate-http-header'>
                <xsl:with-param name='httpHeader' select='.' />
              </xsl:call-template>
            </xsl:for-each>
	    <fault>
	      <xsl:attribute name='localName'>
		<xsl:value-of select='$interfaceFaultComponent/cm:name/cmbase:localName'/>
	      </xsl:attribute>
              <xsl:attribute name='namespace'>
                <xsl:value-of select='$interfaceFaultComponent/cm:name/cmbase:namespaceName'/>
              </xsl:attribute>
              <xsl:if test='$bindingFaultComponent/cmsoap:soapBindingFaultExtension/cmsoap:soapFaultCode'>
                <xsl:choose>
                  <xsl:when test='$bindingFaultComponent/cmsoap:soapBindingFaultExtension/cmsoap:soapFaultCode/cmsoap:code'>
                    <soapFaultCode type='code'
                                   localName='{$bindingFaultComponent/cmsoap:soapBindingFaultExtension/cmsoap:soapFaultCode/cmsoap:code/cmbase:localName}'
                                   namespace='{$bindingFaultComponent/cmsoap:soapBindingFaultExtension/cmsoap:soapFaultCode/cmsoap:code/cmbase:namespaceName}' />
                  </xsl:when>
                  <xsl:otherwise>
                    <soapFaultCode type='#any' />
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:if>
              <xsl:if test='$bindingFaultComponent/cmsoap:soapBindingFaultExtension/cmsoap:soapFaultCode'>
                <xsl:choose>
                  <xsl:when test='$bindingFaultComponent/cmsoap:soapBindingFaultExtension/cmsoap:soapFaultSubcodes/cmsoap:subcodes/cmsoap:code'>
                    <xsl:for-each select='$bindingFaultComponent/cmsoap:soapBindingFaultExtension/cmsoap:soapFaultSubcodes/cmsoap:subcodes/cmsoap:code'>
                      <soapFaultSubcode type='code'
                                        localName='{cmbase:localName}'
                                        namespace='{cmbase:namespaceName}' />
                    </xsl:for-each>
                  </xsl:when>
                  <xsl:otherwise>
                    <soapFaultSubcode type='#any' />
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:if>
              <contentModel>
              	<xsl:variable name='elementDeclarationComponent' select='//cm:elementDeclarationComponent[@xml:id = $interfaceFaultComponent/cm:elementDeclaration/@ref]'/>
                <xsl:attribute name='localName'>
                  <xsl:value-of select='$elementDeclarationComponent/cm:name/cmbase:localName'/>
                </xsl:attribute>
                <xsl:attribute name='namespace'>
                  <xsl:value-of select='$elementDeclarationComponent/cm:name/cmbase:namespaceName'/>
                </xsl:attribute>
              </contentModel>
            </fault>          
          </xsl:when>
          <xsl:otherwise>
            <xsl:variable name='msgCModel' select='$interfaceMessageReferenceComponent/cm:messageContentModel'/>              
            <messageContentModel type='{$msgCModel}'>
              <xsl:if test='$msgCModel="#element"'>
                <xsl:variable name='elementDeclarationComponent' select='//cm:elementDeclarationComponent[@xml:id = $interfaceMessageReferenceComponent/cm:elementDeclaration/@ref]'/>
                <xsl:attribute name='localName'>
                  <xsl:value-of select='$elementDeclarationComponent/cm:name/cmbase:localName'/>
                </xsl:attribute>
                <xsl:attribute name='namespace'>
                  <xsl:value-of select='$elementDeclarationComponent/cm:name/cmbase:namespaceName'/>
                </xsl:attribute>
              </xsl:if>
            </messageContentModel>          
          </xsl:otherwise>
        </xsl:choose>
      </xsl:if>
    </xsl:when>
    <xsl:when test='$bindingComponent/cm:type="http://www.w3.org/ns/wsdl/http"'>      
      <httpVersion>1.1</httpVersion>
      <xsl:if test='$bindingComponent/cmhttp:httpBindingExtension/cmhttp:httpCookies'>
        <httpCookies>
          <xsl:value-of select='$bindingComponent/cmhttp:httpBindingExtension/cmhttp:httpCookies'/>
        </httpCookies>
      </xsl:if>
      <xsl:if test='$bindingComponent/cmhttp:httpBindingExtension/cmhttp:httpQueryParameterSeparatorDefault'>
        <httpQueryParameterSeparatorDefault>
          <xsl:value-of select='$bindingComponent/cmhttp:httpBindingExtension/cmhttp:httpQueryParameterSeparatorDefault'/>
        </httpQueryParameterSeparatorDefault>
      </xsl:if>
      <xsl:if test='$bindingOperationComponent/cmhttp:httpBindingOperationExtension/cmhttp:httpLocation'>
        <httpLocation>
          <xsl:call-template name="replaceHttpLocation">
            <xsl:with-param name="location" select='$bindingOperationComponent/cmhttp:httpBindingOperationExtension/cmhttp:httpLocation' />
          </xsl:call-template>
        </httpLocation>
      </xsl:if>
      <xsl:if test='$bindingOperationComponent/cmhttp:httpBindingOperationExtension/cmhttp:httpLocationIgnoreUncited'>
        <httpLocationIgnoreUncited>
	  <xsl:value-of select='$bindingOperationComponent/cmhttp:httpBindingOperationExtension/cmhttp:httpLocationIgnoreUncited' />
	</httpLocationIgnoreUncited>
      </xsl:if>
      <xsl:if test='$bindingOperationComponent/cmhttp:httpBindingOperationExtension/cmhttp:httpInputSerialization'>
        <httpInputSerialization>
          <xsl:value-of select='$bindingOperationComponent/cmhttp:httpBindingOperationExtension/cmhttp:httpInputSerialization'/>
        </httpInputSerialization>
      </xsl:if>
      <xsl:if test='$bindingOperationComponent/cmhttp:httpBindingOperationExtension/cmhttp:httpOutputSerialization'>
        <httpOutputSerialization>
          <xsl:value-of select='$bindingOperationComponent/cmhttp:httpBindingOperationExtension/cmhttp:httpOutputSerialization'/>
        </httpOutputSerialization>
      </xsl:if>
      <xsl:if test='$bindingOperationComponent/cmhttp:httpBindingOperationExtension/cmhttp:httpFaultSerialization'>
        <httpFaultSerialization>
          <xsl:value-of select='$bindingOperationComponent/cmhttp:httpBindingOperationExtension/cmhttp:httpFaultSerialization'/>
        </httpFaultSerialization>
      </xsl:if>
      <xsl:choose>
	<xsl:when test='$bindingOperationComponent/cmhttp:httpBindingOperationExtension/cmhttp:httpMethod'>
	  <httpMethod>
	    <xsl:value-of select='$bindingOperationComponent/cmhttp:httpBindingOperationExtension/cmhttp:httpMethod' />
	  </httpMethod>
	</xsl:when>
        <xsl:when test='$bindingComponent/cmhttp:httpBindingExtension/cmhttp:httpMethodDefault'>
          <httpMethod>
            <xsl:value-of select='$bindingComponent/cmhttp:httpBindingExtension/cmhttp:httpMethodDefault'/>
          </httpMethod>
        </xsl:when>
      </xsl:choose>
      <xsl:choose>
        <xsl:when test='$bindingMessageReferenceComponent/cmhttp:httpBindingMessageReferenceExtension/cmhttp:httpContentEncoding'>
          <httpContentEncoding>
            <xsl:value-of select='$bindingMessageReferenceComponent/cmhttp:httpBindingMessageReferenceExtension/cmhttp:httpContentEncoding'/>
          </httpContentEncoding>
        </xsl:when>
        <xsl:when test='$bindingOperationComponent/cmhttp:httpBindingOperationExtension/cmhttp:httpContentEncodingDefault'>
          <httpContentEncoding>
            <xsl:value-of select='$bindingOperationComponent/cmhttp:httpBindingOperationExtension/cmhttp:httpContentEncodingDefault'/>
          </httpContentEncoding>
        </xsl:when>
        <xsl:when test='$bindingComponent/cmhttp:httpBindingExtension/cmhttp:httpContentEncodingDefault'>
          <httpContentEncoding>
            <xsl:value-of select='$bindingComponent/cmhttp:httpBindingExtension/cmhttp:httpContentEncodingDefault'/>
          </httpContentEncoding>
        </xsl:when>
      </xsl:choose>
      <xsl:if test='$endpointComponent/cmhttp:httpEndpointExtension/cmhttp:httpAuthenticationRealm'>
        <httpAuthenticationRealm><xsl:value-of select='$endpointComponent/cmhttp:httpEndpointExtension/cmhttp:httpAuthenticationRealm' /></httpAuthenticationRealm>
      </xsl:if>
      <xsl:if test='$endpointComponent/cmhttp:httpEndpointExtension/cmhttp:httpAuthenticationScheme'>
        <httpAuthenticationScheme><xsl:value-of select='$endpointComponent/cmhttp:httpEndpointExtension/cmhttp:httpAuthenticationScheme' /></httpAuthenticationScheme>
      </xsl:if>
      <xsl:if test='$interfaceMessageReferenceComponent'>
        <xsl:for-each select='$bindingMessageReferenceComponent/cmhttp:httpBindingMessageReferenceExtension/cmhttp:httpHeaders/cmhttp:httpHeaderComponent'>
          <xsl:call-template name='generate-http-header'>
            <xsl:with-param name='httpHeader' select='.' />
          </xsl:call-template>
        </xsl:for-each>
      </xsl:if>

      <xsl:choose>
	<xsl:when test='local-name($interfaceMessageReferenceComponent)="interfaceFaultReferenceComponent"'>
	  <xsl:for-each select='$bindingFaultComponent/cmhttp:httpBindingFaultExtension/cmhttp:httpHeaders/cmhttp:httpHeaderComponent'>
	    <xsl:call-template name='generate-http-header'>
	      <xsl:with-param name='httpHeader' select='.' />
	    </xsl:call-template>
	  </xsl:for-each>
	  <fault>
	    <xsl:attribute name='localName'>
	      <xsl:value-of select='$interfaceFaultComponent/cm:name/cmbase:localName'/>
	    </xsl:attribute>
	    <xsl:attribute name='namespace'>
	      <xsl:value-of select='$interfaceFaultComponent/cm:name/cmbase:namespaceName'/>
	    </xsl:attribute>
	    <xsl:if test='$bindingFaultComponent/cmhttp:httpBindingFaultExtension/cmhttp:httpErrorStatusCode'>
	      <xsl:choose>
    		<xsl:when test='$bindingFaultComponent/cmhttp:httpBindingFaultExtension/cmhttp:httpErrorStatusCode/cmhttp:code'>
    		  <xsl:for-each select='$bindingFaultComponent/cmhttp:httpBindingFaultExtension/cmhttp:httpErrorStatusCode/cmhttp:code'>
    		    <httpErrorStatusCode type='{.}'/>
    		  </xsl:for-each>
    		</xsl:when>
    		<xsl:otherwise>
    		  <httpErrorStatusCode type='#any' />
    		</xsl:otherwise>
	      </xsl:choose>
	    </xsl:if>
	    <contentModel>
	      <xsl:variable name='elementDeclarationComponent' select='//cm:elementDeclarationComponent[@xml:id = $interfaceFaultComponent/cm:elementDeclaration/@ref]'/>
	      <xsl:attribute name='localName'>
		<xsl:value-of select='$elementDeclarationComponent/cm:name/cmbase:localName'/>
	      </xsl:attribute>
	      <xsl:attribute name='namespace'>
		<xsl:value-of select='$elementDeclarationComponent/cm:name/cmbase:namespaceName'/>
	      </xsl:attribute>
	    </contentModel>
	    <!-- @@@ -->
	    <xsl:choose>
	      <xsl:when test='$bindingFaultComponent/cmhttp:httpBindingFaultExtension/cmhttp:httpContentEncoding'>
	        <httpContentEncoding>
	          <xsl:value-of select='$bindingFaultComponent/cmhttp:httpBindingFaultExtension/cmhttp:httpContentEncoding'/>
	        </httpContentEncoding>
	      </xsl:when>
	      <xsl:when test='$bindingOperationComponent/cmsoap:soapBindingOperationExtension/cmhttp:httpContentEncodingDefault'>
	        <httpContentEncoding>
	          <xsl:value-of select='$bindingOperationComponent/cmsoap:soapBindingOperationExtension/cmhttp:httpContentEncodingDefault'/>
	        </httpContentEncoding>
	      </xsl:when>
	    </xsl:choose>
	  </fault>          
	</xsl:when>
	<xsl:otherwise>
	  <xsl:variable name='msgCModel' select='$interfaceMessageReferenceComponent/cm:messageContentModel'/>              
	  <messageContentModel type='{$msgCModel}'>
	    <xsl:if test='$msgCModel="#element"'>
	      <xsl:variable name='elementDeclarationComponent' select='//cm:elementDeclarationComponent[@xml:id = $interfaceMessageReferenceComponent/cm:elementDeclaration/@ref]'/>
	      <xsl:attribute name='localName'>
		<xsl:value-of select='$elementDeclarationComponent/cm:name/cmbase:localName'/>
	      </xsl:attribute>
	      <xsl:attribute name='namespace'>
		<xsl:value-of select='$elementDeclarationComponent/cm:name/cmbase:namespaceName'/>
	      </xsl:attribute>
	    </xsl:if>
	  </messageContentModel>          
	</xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    </xsl:choose>
  </xsl:template>

  <xsl:template name='generate-soap-header'>
    <xsl:param name='soapHeader'/>
    <xsl:param name='elementDeclarationComponent'/>
    
    <soapHeader mustUnderstand='{$soapHeader/cmsoap:mustUnderstand}' required='{$soapHeader/cmsoap:required}'>
      <xsl:attribute name='localName'>
        <xsl:value-of select='$elementDeclarationComponent/cm:name/cmbase:localName'/>
      </xsl:attribute>
      <xsl:attribute name='namespace'>
        <xsl:value-of select='$elementDeclarationComponent/cm:name/cmbase:namespaceName'/>
      </xsl:attribute>
    </soapHeader>
  </xsl:template>
  
  <xsl:template name='generate-http-header'>
    <xsl:param name='httpHeader'/>
    <xsl:param name='elementDeclarationComponent'/>
    
    <httpHeader name='{$httpHeader/cmhttp:name}'
                required='{$httpHeader/cmhttp:required}'
                type='{$httpHeader/cmhttp:typeDefinition/@ref}'/>
  </xsl:template>
  
  <!-- Returns the number of messages from a given list of interface[@ref] -->
  <xsl:template name='count-messages-in-interfaceReferences'>
    <!-- list of interface[@ref] -->
    <xsl:param name='interfaceReferences' />
    
    <xsl:choose>
      <xsl:when test='count($interfaceReferences) = 0'>
        <xsl:text>0</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name='interfaceComponent'
                      select='//cm:interfaceComponent[@xml:id = $interfaceReferences[position()=1]/@ref]' />

        <xsl:variable name='extended'>
          <xsl:call-template name='count-messages-in-interfaceReferences'>
            <xsl:with-param name='interfaceReferences' select='$interfaceComponent/cm:extendedInterfaces/cm:interface'/>
          </xsl:call-template>
        </xsl:variable>

        <xsl:variable name='current'>
          <xsl:call-template name='count-messages-in-interfaceOperationComponents'>
            <xsl:with-param name='interfaceOperationComponents'
                            select='$interfaceComponent/cm:interfaceOperations/cm:interfaceOperationComponent'/>
          </xsl:call-template>
        </xsl:variable>
        <xsl:variable name='rest'>
          <xsl:call-template name='count-messages-in-interfaceReferences'>
            <xsl:with-param name='interfaceReferences' select='$interfaceReferences[position()>1]'/>
          </xsl:call-template>
        </xsl:variable>
        <xsl:value-of select='$extended + $current + $rest'/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name='count-messages-in-interfaceOperationComponents'>
    <xsl:param name='interfaceOperationComponents' />
    <xsl:choose>
      <xsl:when test='count($interfaceOperationComponents) = 0'>
        <xsl:text>0</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:variable name='rest'>
          <xsl:call-template name='count-messages-in-interfaceOperationComponents'>
            <xsl:with-param name='interfaceOperationComponents' select='$interfaceOperationComponents[position()>1]'/>
          </xsl:call-template>
        </xsl:variable>
        <xsl:value-of select='2 + $rest'/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>
  
  <xsl:template name="replaceHttpLocation">
    <xsl:param name="location"/>
    <xsl:variable name="httpLocationTable" select="document('httpLocationTable.xml')"/>
    <xsl:choose>
      <xsl:when test="$httpLocationTable/httpLocationTable/httpLocation[in-property = $location]">
      	<xsl:for-each select="$httpLocationTable/httpLocationTable/httpLocation[in-property = $location]/in-message">
        	<alternate><xsl:value-of select="."/></alternate>
      	</xsl:for-each>
      </xsl:when>
      <xsl:otherwise>
        <alternate><xsl:value-of select="$location"/></alternate>        
      </xsl:otherwise>      
    </xsl:choose>
  </xsl:template>
</xsl:stylesheet>
