<?xml version="1.0" encoding="utf-8"?>
<!--

    $Header: /sources/public/2002/ws/desc/test-suite/results-messages/generate-xpath-assertions.xslt,v 1.25 2007/03/23 05:50:53 jonathanMarsh Exp $

    Convert WSDL Component model into XPath expressions

-->
<xsl:stylesheet version="1.0" xmlns="http://www.w3.org/2006/11/desc/assertions"
	xmlns:m="http://www.w3.org/2006/11/desc/messages" xmlns:l="http://www.w3.org/2002/ws/desc/logs/"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:module="http://www.w3.org/2002/ws/desc/6/10/test-module"
	xmlns:soap12="http://www.w3.org/2003/05/soap-envelope"
	xmlns:soap11="http://schemas.xmlsoap.org/soap/envelope/">

	<xsl:output method="xml" indent="yes"/>

	<xsl:template match="/">
		<assertionSets>
			<xsl:apply-templates select="/m:messages/m:message"/>
		</assertionSets>
	</xsl:template>

	<xsl:template match="m:message">
		<assertionSet messagePosition="{@messagePosition}">
			<!-- Copy all information about a message -->
			<xsl:for-each select=".">
				<xsl:copy>
					<xsl:apply-templates select="@*|*"/>
				</xsl:copy>
			</xsl:for-each>
			
			<!-- Now starts the generation of the XPath expressions -->
			<xsl:variable name="msg">/l:log/l:message[@number="<xsl:value-of
					select="@messagePosition"/>"]</xsl:variable>
			<assertion><xsl:value-of select="$msg"/>/@type="<xsl:value-of select="@type"/>"</assertion>
			
			<!-- Tests common to HTTP and SOAP bindings -->
			<xsl:if test="@type='request'">
				<xsl:if test="@type='request' and m:httpLocation">
					<assertion>
						<xsl:for-each select="m:httpLocation/m:alternate">
							<xsl:text>contains(</xsl:text>
							<xsl:value-of select="$msg"/>
							<xsl:text>/l:httpHeaders/l:Method/@rawURI, "</xsl:text>
							<xsl:value-of select="."/>
							<xsl:text>")</xsl:text>
							<xsl:if test="position() != last()"> or </xsl:if>
						</xsl:for-each>
					</assertion>
				</xsl:if>
			</xsl:if>

			<xsl:for-each select="m:httpHeader[@required='true']">
				<assertion>
					<xsl:value-of select="$msg"/>
					<xsl:text>/l:httpHeaders/l:</xsl:text>
					<xsl:value-of select="@name"/>
				</assertion>
			</xsl:for-each>

			<!-- Tests specific to HTTP 1.1 bindings -->
			<xsl:if test="m:httpVersion='1.1'">
				<xsl:choose>
					<xsl:when test="m:fault">
						<assertion>
							<xsl:value-of select="$msg"/>
							<xsl:text>/l:httpHeaders/l:Content-Type/@type='</xsl:text>
							<xsl:value-of select="m:httpFaultSerialization"/>
							<xsl:text>'</xsl:text>
						</assertion>
					</xsl:when>
					<xsl:when test="@type='request' and 
									m:httpMethod != 'GET' and
									m:httpMethod != 'DELETE' and (
										m:messageContentModel/@type!='#none' or
										m:httpInputSerialization='application/x-www-form-urlencoded'
									)">
						<assertion>
							<xsl:value-of select="$msg"/>
							<xsl:text>/l:httpHeaders/l:Content-Type/@type='</xsl:text>
							<xsl:value-of select="m:httpInputSerialization"/>
							<xsl:text>'</xsl:text>
						</assertion>
					</xsl:when>
					<xsl:when test="@type='response' and
									m:messageContentModel/@type!='#none' and
									m:operation/@mep='http://www.w3.org/ns/wsdl/in-out'">
						<assertion>
							<xsl:value-of select="$msg"/>
							<xsl:text>/l:httpHeaders/l:Content-Type/@type='</xsl:text>
							<xsl:value-of select="m:httpOutputSerialization"/>
							<xsl:text>'</xsl:text>
						</assertion>
					</xsl:when>
				</xsl:choose>
				<xsl:if test="@type='request'">
					<assertion>
						<xsl:value-of select="$msg"/>
						<xsl:text>/l:httpHeaders/l:Method/@value='</xsl:text>
						<xsl:choose>
							<xsl:when test="m:httpMethod">
								<xsl:value-of select="m:httpMethod"/>
							</xsl:when>
							<xsl:when test="m:safety='true'">
								<xsl:text>GET</xsl:text>
							</xsl:when>
							<xsl:otherwise>
								<xsl:text>POST</xsl:text>
							</xsl:otherwise>
						</xsl:choose>
						<xsl:text>'</xsl:text>
					</assertion>
				</xsl:if>
				<xsl:if test="@type='response'">
					<xsl:choose>
						<xsl:when test="m:fault/m:httpErrorStatusCode[@type!='#any']">
							<assertion>
								<xsl:value-of select="$msg"/>
								<xsl:text>/l:httpHeaders/l:StatusCode/@value='</xsl:text>
								<!-- @@There could be more than one HTTP Error status code -->
								<xsl:value-of select="m:fault/m:httpErrorStatusCode/@type"/>
								<xsl:text>'</xsl:text>
							</assertion>
						</xsl:when>
						<xsl:when test="m:fault">
							<assertion>
								<xsl:value-of select="$msg"/>
								<xsl:text>/l:httpHeaders/l:StatusCode/@value='500'</xsl:text>
							</assertion>
						</xsl:when>
						<xsl:when test="m:fault/m:httpErrorStatusCode[@type='#any']">
							<assertion><xsl:value-of select="$msg"/>/l:httpHeaders/l:StatusCode/@value='500'</assertion>
						</xsl:when>
						<xsl:when
							test="m:operation/@mep='http://www.w3.org/ns/wsdl/in-out'">
							<assertion><xsl:value-of select="$msg"/>/l:httpHeaders/l:StatusCode/@value='200'</assertion>
						</xsl:when>
						<xsl:when
							test="m:operation/@mep='http://www.w3.org/ns/wsdl/in-only'">
							<assertion><xsl:value-of select="$msg"/>/l:httpHeaders/l:StatusCode/@value='202'</assertion>
						</xsl:when>
						<xsl:when
							test="m:operation/@mep='http://www.w3.org/ns/wsdl/robust-in-only'">
							<assertion><xsl:value-of select="$msg"/>/l:httpHeaders/l:StatusCode/@value='204'</assertion>
						</xsl:when>
					</xsl:choose>
				</xsl:if>
				<xsl:if test="(@type='request')         or
					(@type='response'          and
					m:operation/@mep='http://www.w3.org/ns/wsdl/in-out')         or
					m:fault">
					<!-- @@it's a GET, no body... -->
					<xsl:choose>
						<xsl:when test="not(m:fault) and m:httpContentEncoding!=''">
							<assertion><xsl:value-of select="$msg"/>/l:httpHeaders/l:Content-Encoding='<xsl:value-of
									select="m:httpContentEncoding"/>'</assertion>
						</xsl:when>
						<xsl:when test="m:fault/m:httpContentEncoding!=''">
							<assertion><xsl:value-of select="$msg"/>/l:httpHeaders/l:Content-Encoding='<xsl:value-of
									select="m:fault/m:httpContentEncoding"/>'</assertion>
						</xsl:when>
						<xsl:when test="m:fault">
							<assertion>count(<xsl:value-of select="$msg"/>/l:content/*)=1</assertion>
							<xsl:if test="m:fault/m:contentModel">
								<assertion><xsl:value-of select="$msg"/><xsl:text>/l:content/*[local-name()="</xsl:text>
									<xsl:value-of select="m:fault/m:contentModel/@localName"/>" and namespace-uri()="<xsl:value-of
										select="m:fault/m:contentModel/@namespace"/>"]</assertion>
							</xsl:if>
						</xsl:when>
						<xsl:when test="m:messageContentModel/@type='#none' or (
											@type='request' and
											m:httpInputSerialization='application/x-www-form-urlencoded'
										)">
							<assertion>count(<xsl:value-of select="$msg"/>/l:content/*)=0</assertion>
						</xsl:when>
						<xsl:when test="@type='request' and
										m:httpInputSerialization='multipart/form-data'">
							<assertion>count(<xsl:value-of select="$msg"/>/l:content/text())&gt;0</assertion>
						</xsl:when>
						<xsl:when test="m:messageContentModel/@type='#any'">
							<!-- it's anything, just check the number.
							   is it really one or can it be more?
						      -->
							<assertion>count(<xsl:value-of select="$msg"/>/l:content/*)&lt;=1</assertion>
						</xsl:when>
						<xsl:when test="m:messageContentModel/@type='#other'"> </xsl:when>
						<xsl:when test="m:messageContentModel/@type='#element'">
							<assertion>count(<xsl:value-of select="$msg"/>/l:content/*)=1</assertion>
							<assertion>
								<xsl:value-of select="$msg"/>
								<xsl:text>/l:content/*[local-name()='</xsl:text>
								<xsl:value-of select="m:messageContentModel/@localName"/>
								<xsl:text>' and namespace-uri()='</xsl:text>
								<xsl:value-of select="m:messageContentModel/@namespace"/>
								<xsl:text>']</xsl:text>
							</assertion>
						</xsl:when>
					</xsl:choose>
				</xsl:if>
			</xsl:if>

			<!-- Tests specific to SOAP 1.2 bindings -->
			<xsl:if test="m:soapVersion='1.2'">
				<xsl:if test="@type='request'">
					<xsl:choose>
						<xsl:when test="m:soapMep='http://www.w3.org/2003/05/soap/mep/soap-response/'">
							<assertion><xsl:value-of select="$msg"/>/l:httpHeaders/l:Method/@value='GET'</assertion>
							<xsl:if test="m:soapAction">
								<assertion><xsl:value-of select="$msg"/>/l:httpHeaders[not(l:Content-Type/l:action)]</assertion>
							</xsl:if>
						</xsl:when>
						<xsl:otherwise>
							<assertion><xsl:value-of select="$msg"/>/l:httpHeaders/l:Method/@value='POST'</assertion>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:if>
				<xsl:if test="@type='response'">
					<xsl:choose>
						<xsl:when test="m:fault/m:soapFaultCode[
											@type='code' and
											@localName='Sender' and
											@namespace='http://www.w3.org/2003/05/soap-envelope'
										]">
							<assertion><xsl:value-of select="$msg"/>/l:httpHeaders/l:StatusCode/@value='400'</assertion>
						</xsl:when>
						<xsl:when test="m:fault/m:soapFaultCode[
											@type='code' and
											@localName='Receiver' and
											@namespace='http://www.w3.org/2003/05/soap-envelope'
										]">
							<assertion><xsl:value-of select="$msg"/>/l:httpHeaders/l:StatusCode/@value='500'</assertion>
						</xsl:when>
						<xsl:when test="m:fault">
							<!--    @@ No assertion as to the StatusCode on a fault with soapFaultCode = #any @@
	      <assertion><xsl:value-of select='$msg'/>/l:httpHeaders/l:StatusCode/@value='500'</assertion>
-->
						</xsl:when>
						<xsl:when
							test="m:operation/@mep='http://www.w3.org/ns/wsdl/in-out'">
							<assertion><xsl:value-of select="$msg"/>/l:httpHeaders/l:StatusCode/@value='200'</assertion>
						</xsl:when>
						<xsl:when test="m:operation/@mep='http://www.w3.org/ns/wsdl/in-only' or
										m:operation/@mep='http://www.w3.org/ns/wsdl/robust-in-only'">
							<assertion><xsl:value-of select="$msg"/>/l:httpHeaders/l:StatusCode/@value='202'</assertion>
						</xsl:when>
					</xsl:choose>
				</xsl:if>
				<xsl:if test="(@type='request' and m:soapMep!='http://www.w3.org/2003/05/soap/mep/soap-response/') or
							  (@type='response' and m:operation/@mep='http://www.w3.org/ns/wsdl/in-out')">
					<assertion><xsl:value-of select="$msg" />/l:httpHeaders/l:Content-Type/@type='application/soap+xml'</assertion>
					<xsl:if test="m:soapAction">
						<assertion><xsl:value-of select="$msg"/>/l:httpHeaders/l:Content-Type/l:action="<xsl:value-of
								select="m:soapAction"/>"</assertion>
					</xsl:if>
					<xsl:choose>
						<xsl:when test="m:httpContentEncoding!=''">
							<assertion><xsl:value-of select="$msg"/>/l:httpHeaders/l:Content-Encoding='<xsl:value-of
									select="m:httpContentEncoding"/>'</assertion>
						</xsl:when>
						<xsl:otherwise>
							<assertion><xsl:value-of select="$msg"/>/l:content/soap12:Envelope</assertion>
						</xsl:otherwise>
					</xsl:choose>
				</xsl:if>
				<xsl:if test="m:httpContentEncoding='' or not(m:httpContentEncoding)">
					<xsl:if test="m:fault">
						<assertion>count(<xsl:value-of select="$msg"/>/l:content/soap12:Envelope/soap12:Body/*)=1</assertion>
						<assertion><xsl:value-of select="$msg"/>/l:content/soap12:Envelope/soap12:Body/soap12:Fault</assertion>
						<xsl:if test="m:fault/m:soapFaultCode[@type='code']">
							<assertion>
								<xsl:value-of select="$msg"/>
								<xsl:text>/l:content/soap12:Envelope/soap12:Body/soap12:Fault/soap12:Code/soap12:Value[substring-after(.,":")="</xsl:text>
								<xsl:value-of select="m:fault/m:soapFaultCode/@localName"/>
								<xsl:text>" or text()="</xsl:text>
								<xsl:value-of select="m:fault/m:soapFaultCode/@localName"/>
								<xsl:text>"][namespace::*[.="</xsl:text>
								<xsl:value-of select="m:fault/m:soapFaultCode/@namespace"/>
								<xsl:text>" and local-name()=substring-before(..,":")</xsl:text>
								<xsl:text>]]</xsl:text>
							</assertion>
						</xsl:if>
						<xsl:for-each select="m:fault/m:soapFaultSubcode[@type='code']">
							<assertion>
								<xsl:value-of select="$msg"/>
								<xsl:text>/l:content/soap12:Envelope/soap12:Body/soap12:Fault/soap12:Code//soap12:Subcode/soap12:Value[substring-after(.,":")="</xsl:text>
								<xsl:value-of select="@localName"/>
								<xsl:text>" or text()="</xsl:text>
								<xsl:value-of select="@localName"/>
								<xsl:text>"][namespace::*[.="</xsl:text>
								<xsl:value-of select="@namespace"/>
								<xsl:text>" and local-name()=substring-before(..,":")</xsl:text>
								<xsl:text>]]</xsl:text>
							</assertion>
						</xsl:for-each>
						<xsl:if test="m:fault/m:contentModel">
							<assertion><xsl:value-of select="$msg"/>/l:content/soap12:Envelope/soap12:Body/soap12:Fault/soap12:Detail/*[<xsl:text>local-name()="</xsl:text>
								<xsl:value-of select="m:fault/m:contentModel/@localName"/>" and
									namespace-uri()="<xsl:value-of select="m:fault/m:contentModel/@namespace"/>"]</assertion>
						</xsl:if>
					</xsl:if>
					<xsl:if test="m:soapModule/@ref='http://www.w3.org/2002/ws/desc/6/10/test-module'">
						<assertion><xsl:value-of select="$msg"/>/l:content/soap12:Envelope/soap12:Header/module:module-test</assertion>
					</xsl:if>
					<xsl:for-each select="m:soapHeader[@required='true']">
						<assertion>
							<xsl:value-of select="$msg"/>
							<xsl:text>/l:content/soap12:Envelope/soap12:Header/*[</xsl:text>
							<xsl:if test="@mustUnderstand='true'">(@soap12:mustUnderstand="true" or @soap12:mustUnderstand="1") and </xsl:if>
							<xsl:text>local-name()="</xsl:text>
							<xsl:value-of select="@localName"/>" and namespace-uri()="<xsl:value-of
								select="@namespace"/>"]</assertion>
					</xsl:for-each>
					<xsl:if test="(@type='request' and m:soapMep!='http://www.w3.org/2003/05/soap/mep/soap-response/') or
								  (@type='response' and count(m:fault)=0 and m:operation/@mep='http://www.w3.org/ns/wsdl/in-out')">
						<xsl:choose>
							<xsl:when test="m:messageContentModel/@type='#any'">
								<!-- it's anything, just check the number.
								     is it really one or can it be more?
								-->
								<assertion>count(<xsl:value-of select="$msg"/>/l:content/soap12:Envelope/soap12:Body/*)&lt;=1</assertion>
							</xsl:when>
							<xsl:when test="m:messageContentModel/@type='#other'"> </xsl:when>
							<xsl:when test="m:messageContentModel/@type='#none'">
								<assertion>count(<xsl:value-of select="$msg"/>/l:content/soap12:Envelope/soap12:Body/*)=0</assertion>
							</xsl:when>
							<xsl:when test="m:messageContentModel/@type='#element'">
								<assertion>count(<xsl:value-of select="$msg"/>/l:content/soap12:Envelope/soap12:Body/*)=1</assertion>
								<assertion>
									<xsl:value-of select="$msg"/>
									<xsl:text>/l:content/soap12:Envelope/soap12:Body/*[local-name()='</xsl:text>
									<xsl:value-of select="m:messageContentModel/@localName"/>
									<xsl:text>' and namespace-uri()='</xsl:text>
									<xsl:value-of select="m:messageContentModel/@namespace"/>
									<xsl:text>']</xsl:text>
								</assertion>
							</xsl:when>
						</xsl:choose>
					</xsl:if>
				</xsl:if>
			</xsl:if>
		</assertionSet>
	</xsl:template>

	<xsl:template match="@*|*|text()">
		<xsl:copy>
			<xsl:apply-templates select="@*|*|text()"/>
		</xsl:copy>
	</xsl:template>
</xsl:stylesheet>
