<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns="http://www.w3.org/2002/ws/desc/logs/"
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
    <xsl:output method="html" encoding="UTF-8" indent="no"/>
    <xsl:param name="DSTAMP"/>
    <xsl:param name="TSTAMP"/>
    <xsl:param name="TODAY"/>
    <xsl:template match="/">
        <html>
            <head>
                <title>Message Exchange Test Results</title>
                <style type="text/css">
                    body {font-family:sans-serif}
                    .passed {background-color:green}
                    .failed {background-color:red}
                </style>
            </head>
            <body>
                <h1>Message Exchange Test Results</h1>
                <xsl:if test="$TODAY">
                    <address>
                        <xsl:text>Generated </xsl:text>
                        <xsl:value-of select="$TODAY"/>,
                        <xsl:value-of select="$TSTAMP"/>
                    </address>
                </xsl:if>
                <p>Results of message exchange tests between client and service based on WSDL 2.0 documents.</p>
                <xsl:call-template name="message-exchange-results-summary"/>
                <xsl:for-each select="message-test-summary/testpass">
                    <xsl:variable name="testpass" select="."/>
                    <h3 id="{@id}"><xsl:value-of select="@id"/> tests (<xsl:value-of select="@binding"/> binding)</h3>
                    <table>
                        <xsl:apply-templates select="../implementations" mode="headings"/>
                        <tbody>
                            <tr>
                                <td/>
						        <xsl:for-each select="../implementations/implementation">
						        	<xsl:variable name="client" select="@id"/>
				                    <xsl:for-each select="../implementation">
					                    <xsl:variable name="service" select="@id"/>
						            	<xsl:if test="count($testpass/logs/log[@service = $service][@client = $client]) = 0">
						            		<td/>
						            	</xsl:if>
						            	<xsl:for-each select="$testpass/logs/log[@service = $service][@client = $client]">
    										<xsl:call-template name="message-exchange-results"/>
                                        </xsl:for-each>
						            </xsl:for-each>
			        	        </xsl:for-each>
                            </tr>
                            <tr/>
                            <xsl:for-each select="test">
                                <xsl:variable name="test" select="."/>
							    <tr>
                                    <td>
                                        <xsl:value-of select="@number"/>
                                    </td>
							        <xsl:for-each select="../../implementations/implementation">
							        	<xsl:variable name="client" select="@id"/>
					                    <xsl:for-each select="../implementation">
						                    <xsl:variable name="service" select="@id"/>
							            	<xsl:if test="count($testpass/logs/log[@service = $service][@client = $client]) = 0">
							            		<td/>
							            	</xsl:if>
							            	<xsl:for-each select="$testpass/logs/log[@service = $service][@client = $client]">
			                                    <td class="{$test/result[@implementation = current()/@id]}" style="width:150px">
			                                        <a href="{substring-before(@href,'-output.xml')}-results.html#message{$test/@number}">log</a>
			                                    </td>
							                </xsl:for-each>
							            </xsl:for-each>
				        	        </xsl:for-each>
			                    </tr>
                            </xsl:for-each>
                        </tbody>
                    </table>
                </xsl:for-each>
            </body>
        </html>
    </xsl:template>
    <xsl:template match="implementations" mode="headings">
    	<thead>
            <tr>
                <th/>
                <xsl:for-each select="implementation">
                    <xsl:variable name="currentName" select="@name"/>
                    <xsl:for-each select="../implementation">
	                    <th>
	                        <nobr><xsl:value-of select="$currentName"/></nobr>
	                        <xsl:text> &gt;&gt; </xsl:text>
	                        <nobr><xsl:value-of select="@name"/></nobr>
	                    </th>
	                </xsl:for-each>
                </xsl:for-each>
            </tr>
        </thead>
    </xsl:template>    
    <xsl:template name="message-exchange-results">
        <xsl:variable name="width" select="150"/>
        <td style="border:1px solid black; padding:0px; width:{$width}px; height:18px; font-size:x-small; font-weight:bold">
            <xsl:variable name="total" select="count(ancestor::testpass/test)"/>
            <xsl:variable name="passed" select="count(ancestor::testpass/test/result[@implementation=current()/@id][.='passed'])"/>
            <xsl:variable name="failed" select="count(ancestor::testpass/test/result[@implementation=current()/@id][.='failed'])"/>
            <xsl:variable name="percentage-passed" select="$passed div $total"/>
            <xsl:variable name="percentage-failed" select="$failed div $total"/>
            <xsl:if test="$percentage-passed > .01">
                <span style="position:absolute"><xsl:value-of select="$passed"/></span><img src="green.gif" height="16" width="{round($width*$percentage-passed)}" alt="{round($percentage-passed * 100)}% passed"/></xsl:if>
            <xsl:if test="$percentage-failed > .01">
                <span style="position:absolute"><xsl:value-of select="$failed"/></span><img src="red.gif" height="16" width="{$width - round($width*$percentage-passed)}" alt="{round($percentage-failed * 100)}% failed"/></xsl:if>
        </td>
    </xsl:template>
    <xsl:template name="message-exchange-results-summary">
      	<xsl:param name="detail-path"/>
      	<table>
            <xsl:apply-templates select="message-test-summary/implementations" mode="headings"/>
            <tbody>
				<xsl:for-each select="message-test-summary/testpass">
				    <xsl:variable name="testpass" select="."/>
				    <tr>
						<td style="font-size:smaller"><a href="{$detail-path}#{@id}"><xsl:value-of select="@id"/></a> (<xsl:value-of select="@binding"/>)</td>
				        <xsl:for-each select="../implementations/implementation">
				        	<xsl:variable name="client" select="@id"/>
		                    <xsl:for-each select="../implementation">
			                    <xsl:variable name="service" select="@id"/>
				            	<xsl:if test="count($testpass/logs/log[@service = $service][@client = $client]) = 0">
				            		<td style="border:1px solid black; padding:0px; width:150px; height:18px; font-size:x-small; text-align:center">&#160;</td>
				            	</xsl:if>
				            	<xsl:for-each select="$testpass/logs/log[@service = $service][@client = $client]">
				                    <xsl:call-template name="message-exchange-results"/>
				                </xsl:for-each>
				            </xsl:for-each>
	        	        </xsl:for-each>
                    </tr>
			    </xsl:for-each>
            </tbody>
        </table>        
    </xsl:template>
</xsl:stylesheet>
